@@ -469,7 +469,6 @@ _fprint_device_check_for_username (FprintDevice *rdev,
 	char *sender;
 	unsigned long uid;
 	struct passwd *user;
-	char *client_username;
 
 	/* Get details about the current sender, and username/uid */
 	conn = dbus_g_connection_get_connection (fprintd_dbus_conn);
@@ -490,17 +489,16 @@ _fprint_device_check_for_username (FprintDevice *rdev,
 			    "Failed to get information about user UID %lu", uid);
 		return NULL;
 	}
-	client_username = g_strdup (user->pw_name);
 
 	/* The current user is usually allowed to access their
 	 * own data, this should be followed by PolicyKit checks
 	 * anyway */
-	if (username == NULL || *username == '\0' || g_str_equal (username, client_username)) {
+	if (username == NULL || *username == '\0' || g_str_equal (username, user->pw_name)) {
 		if (ret_sender != NULL)
 			*ret_sender = sender;
 		else
 			g_free (sender);
-		return client_username;
+		return g_strdup (user->pw_name);
 	}
 
 	/* If we're not allowed to set a different username,
@@ -751,8 +749,30 @@ static void fprint_device_release(FprintDevice *rdev,
 	}
 }
 
-static void verify_cb(struct fp_dev *dev, int r, struct fp_img *img,
-		      void *user_data)
+static void verify_cb(struct fp_dev *dev,
+		      int            r,
+		      struct fp_img *img,
+		      void          *user_data);
+
+static void
+verify_restart_cb(struct fp_dev *dev,
+		  void          *user_data)
+{
+	struct FprintDevice *rdev = user_data;
+	FprintDevicePrivate *priv = DEVICE_GET_PRIVATE(rdev);
+
+	if (priv->current_action != ACTION_VERIFY)
+		return;
+
+	g_debug("verify_restart_cb: restarting verification");
+	fp_async_verify_start(priv->dev, priv->verify_data, verify_cb, rdev);
+}
+
+static void
+verify_cb(struct fp_dev *dev,
+	  int            r,
+	  struct fp_img *img,
+	  void          *user_data)
 {
 	struct FprintDevice *rdev = user_data;
 	FprintDevicePrivate *priv = DEVICE_GET_PRIVATE(rdev);
@@ -763,8 +783,20 @@ static void verify_cb(struct fp_dev *dev, int r, struct fp_img *img,
 
 	g_debug("verify_cb: result %s (%d)", name, r);
 
-	if (r == FP_VERIFY_NO_MATCH || r == FP_VERIFY_MATCH || r < 0)
-		priv->action_done = TRUE;
+	if (r == FP_VERIFY_RETRY ||
+	    r == FP_VERIFY_RETRY_TOO_SHORT ||
+	    r == FP_VERIFY_RETRY_CENTER_FINGER ||
+	    r == FP_VERIFY_RETRY_REMOVE_FINGER) {
+		g_debug ("verify_cb: stopping current verification to retry");
+		fp_img_free(img);
+		if (fp_async_verify_stop(priv->dev, verify_restart_cb, user_data) == 0) {
+			g_signal_emit(rdev, signals[SIGNAL_VERIFY_STATUS], 0, name, priv->action_done);
+			return;
+		}
+		/* fallthrough to error out if restarting failed */
+	}
+
+	priv->action_done = TRUE;
 	set_disconnected (priv, name);
 	g_signal_emit(rdev, signals[SIGNAL_VERIFY_STATUS], 0, name, priv->action_done);
 	fp_img_free(img);
@@ -775,8 +807,32 @@ static void verify_cb(struct fp_dev *dev, int r, struct fp_img *img,
 	}
 }
 
-static void identify_cb(struct fp_dev *dev, int r,
-			 size_t match_offset, struct fp_img *img, void *user_data)
+static void identify_cb(struct fp_dev *dev,
+			int            r,
+			size_t         match_offset,
+			struct fp_img *img,
+			void          *user_data);
+
+static void
+identify_restart_cb(struct fp_dev *dev,
+		    void          *user_data)
+{
+	struct FprintDevice *rdev = user_data;
+	FprintDevicePrivate *priv = DEVICE_GET_PRIVATE(rdev);
+
+	if (priv->current_action != ACTION_IDENTIFY)
+		return;
+
+	g_debug("identify_restart_cb: restarting identification");
+	fp_async_identify_start (priv->dev, priv->identify_data, identify_cb, rdev);
+}
+
+static void
+identify_cb(struct fp_dev *dev,
+	    int            r,
+	    size_t         match_offset,
+	    struct fp_img *img,
+	    void          *user_data)
 {
 	struct FprintDevice *rdev = user_data;
 	FprintDevicePrivate *priv = DEVICE_GET_PRIVATE(rdev);
@@ -787,8 +843,20 @@ static void identify_cb(struct fp_dev *dev, int r,
 
 	g_debug("identify_cb: result %s (%d)", name, r);
 
-	if (r == FP_VERIFY_NO_MATCH || r == FP_VERIFY_MATCH || r < 0)
-		priv->action_done = TRUE;
+	if (r == FP_VERIFY_RETRY ||
+	    r == FP_VERIFY_RETRY_TOO_SHORT ||
+	    r == FP_VERIFY_RETRY_CENTER_FINGER ||
+	    r == FP_VERIFY_RETRY_REMOVE_FINGER) {
+		g_debug ("identify_cb: stopping current identification to retry");
+		fp_img_free(img);
+		if (fp_async_identify_stop(priv->dev, identify_restart_cb, user_data) == 0) {
+			g_signal_emit(rdev, signals[SIGNAL_VERIFY_STATUS], 0, name, priv->action_done);
+			return;
+		}
+		/* fallthrough to error out if restarting failed */
+	}
+
+	priv->action_done = TRUE;
 	set_disconnected (priv, name);
 	g_signal_emit(rdev, signals[SIGNAL_VERIFY_STATUS], 0, name, priv->action_done);
 	fp_img_free(img);
@@ -992,6 +1060,13 @@ static void fprint_device_verify_stop(FprintDevice *rdev,
 		return;
 	}
 
+	if (r == -EINPROGRESS) {
+		g_debug ("%s stop already in progress",
+			 priv->current_action == ACTION_VERIFY ? "verification" : "identification");
+		dbus_g_method_return(context);
+		r = 0;
+	}
+
 	if (r < 0) {
 		g_set_error(&error, FPRINT_ERROR, FPRINT_ERROR_INTERNAL,
 			"Verify stop failed with error %d", r);
